package org.freecompany.infoset;

import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.channels.ReadableByteChannel;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import org.freecompany.infoset.dom.DomInfosetFactory;
import org.freecompany.infoset.sax.SaxInfosetFactory;
import org.w3c.dom.Document;
import org.xml.sax.XMLReader;

/**
 * Interface representing factory that builds {@link Infoset} instances.  An instance of this interface must be able to
 * construct concrete {@link Infoset} instances from data streams.
 */
public interface InfosetFactory< T> {

	public class Registry {
		public static Map< Class< ?>, InfosetFactory< ?>> map = Collections.synchronizedMap( new HashMap< Class< ?>, InfosetFactory< ?>>());
		static {
			reg( Document.class, DomInfosetFactory.newInstance());
			reg( XMLReader.class, SaxInfosetFactory.newInstance());
		}
		public static < V> void reg( Class< V> clazz, InfosetFactory< V> factory) { map.put( clazz, factory); }
		@SuppressWarnings( "unchecked")
		public static < V> InfosetFactory< V> lookup( Class< V> clazz) { return ( InfosetFactory< V>) map.get( clazz); }
	}

	/**
	 * Creates an infoset that will, when accessed, parse the content from the provided
	 * {@link URL}.  Once parsed, the content will be cached, so anything that wishes to
	 * modify the resulting {@link Document} must clone it first.
	 */
	Infoset< T> create( ReadableByteChannel channel) throws IOException;
	
	/**
	 * Creates an infoset that will, when accessed, parse the content from the provided
	 * {@link URL}.  Once parsed, the content will be cached, so anything that wishes to
	 * modify the resulting {@link Document} must clone it first.
	 */
	Infoset< T> create( InputStream input) throws IOException;
	
	/**
	 * Creates an infoset that will, when accessed, parse the content from the provided
	 * {@link InputStream}.  Once parsed, the content will be cached, so anything that wishes to
	 * modify the resulting {@link Document} must clone it first.
	 */
	Infoset< T> create( URL url) throws IOException;

	/**
	 * Converts an instance of the provided {@link Infoset} type to the native type for this factory.
	 * If the factory does not support the conversion, it should return <code>null</code>.
	 */
	< V> Infoset< V> adaptTo( Class< V> clazz, Infoset< T> infoset);

	/**
	 * Converts an instance of the provided {@link Infoset} type to the native type for this factory.
	 * If the factory does not support the conversion, it should return <code>null</code>.
	 */
	< V> Infoset< T> adaptFrom( Infoset< V> infoset);
}
