package org.freecompany.infoset.sax;

import java.io.InputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.freecompany.infoset.AbstractInfoset;
import org.freecompany.infoset.Infoset;
import org.freecompany.infoset.InfosetFactory;
import org.freecompany.infoset.DefaultInfoset;
import org.freecompany.infoset.InfosetException;
import org.w3c.dom.Document;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.XMLFilterImpl;

import static org.freecompany.infoset.InfosetFactory.Registry.reg;

public class SaxInfosetFactory implements InfosetFactory< XMLReader> {

	public static SaxInfosetFactory newInstance() { return new SaxInfosetFactory(); }

	protected final SAXParserFactory factory = SAXParserFactory.newInstance();

	protected SaxInfosetFactory() {
		factory.setNamespaceAware( true);
	}

	/**
	 * Creates an infoset that will, when accessed, parse the content from the provided
	 * {@link URL}.  Once parsed, the content will be cached, so anything that wishes to
	 * modify the resulting {@link Document} must clone it first.
	 */
	public Infoset< XMLReader> create( final URL url) throws IOException {
		return create( url.openStream());
	}

	/**
	 * Creates an infoset that will, when accessed, parse the content from the provided
	 * {@link URL}.  Once parsed, the content will be cached, so anything that wishes to
	 * modify the resulting {@link Document} must clone it first.
	 */
	public Infoset< XMLReader> create( final ReadableByteChannel channel) throws IOException {
		return create( Channels.newInputStream( channel));
	}

	/**
	 * Creates an infoset that will, when accessed, parse the content from the provided
	 * {@link InputStream}.  Once parsed, the content will be cached, so anything that wishes to
	 * modify the resulting {@link Document} must clone it first.
	 */
	public Infoset< XMLReader> create( final InputStream input) {
		return new AbstractInfoset< XMLReader>( XMLReader.class, this) {
			public synchronized XMLReader get() {
				final XMLFilterImpl filter;
				try {
					synchronized( SaxInfosetFactory.this.factory) {
						filter = new XMLFilterImpl( SaxInfosetFactory.this.factory.newSAXParser().getXMLReader()){
							@Override public void parse( InputSource source) throws SAXException, IOException { parse(( String) null); }
							@Override public void parse( String id) throws SAXException, IOException { super.parse( new InputSource( input)); }
						};
					}
					return filter;
				} catch ( ParserConfigurationException e) {
					throw new InfosetException( e);
				} catch ( SAXException e) {
					throw new InfosetException( e);
				}
			}
		};
	}

	public < V> Infoset< V> adaptTo( final Class< V> clazz, final Infoset< XMLReader> infoset) {
		if ( clazz.equals( Document.class)) {
			// TODO: Convert SAX stream to DOM document.
		}
		return null;
	}

	@SuppressWarnings( "unchecked")
	public < V> Infoset< XMLReader> adaptFrom( final Infoset< V> infoset) {
		if ( infoset.getType().equals( Document.class)) return new DomWrapperInfoset( this, ( Infoset< Document>) infoset);
		return null;
	}
}
